@@ -25,63 +25,93 @@ import (
 	"github.com/walles/moar/twin"
 )
 
+const defaultDarkTheme = "native"
+
+// I decided on a light theme by doing this:
+//
+//	wc -l ../chroma/styles/*.xml|sort|cut -d/ -f4|grep xml|xargs -I XXX grep -Hi background ../chroma/styles/XXX
+//
+// Then I picked tango because it has a lot of lines, a bright background
+// and I like the looks of it.
+const defaultLightTheme = "tango"
+
 var versionString = "Should be set when building, please use build.sh to build"
 
-func printUsageEnvVar(output io.Writer, envVarName string, description string) {
+func printUsageEnvVar(envVarName string, description string, colors twin.ColorType) {
 	value := os.Getenv(envVarName)
 	if len(value) == 0 {
 		return
 	}
 
-	_, _ = fmt.Fprintf(output, "  %s (%s): %s\n",
+	style, err := m.TermcapToStyle(value)
+	if err != nil {
+		bold := twin.StyleDefault.WithAttr(twin.AttrBold).RenderUpdateFrom(twin.StyleDefault, colors)
+		notBold := twin.StyleDefault.RenderUpdateFrom(twin.StyleDefault.WithAttr(twin.AttrBold), colors)
+		fmt.Printf("  %s (%s): %s %s<- Error: %v%s\n",
+			envVarName,
+			description,
+			strings.ReplaceAll(value, "\x1b", "ESC"),
+			bold,
+			err,
+			notBold,
+		)
+		return
+	}
+
+	prefix := style.RenderUpdateFrom(twin.StyleDefault, colors)
+	suffix := twin.StyleDefault.RenderUpdateFrom(style, colors)
+	fmt.Printf("  %s (%s): %s\n",
 		envVarName,
 		description,
-		strings.ReplaceAll(value, "\x1b", "ESC"),
+		prefix+strings.ReplaceAll(value, "\x1b", "ESC")+suffix,
 	)
 }
 
 func printCommandline(output io.Writer) {
-	_, _ = fmt.Fprintln(output, "Commandline: moar", strings.Join(os.Args[1:], " "))
-	_, _ = fmt.Fprintf(output, "Environment: MOAR=\"%v\"\n", os.Getenv("MOAR"))
-	_, _ = fmt.Fprintln(output)
+	fmt.Fprintln(output, "Commandline: moar", strings.Join(os.Args[1:], " "))
+	fmt.Fprintf(output, "Environment: MOAR=\"%v\"\n", os.Getenv("MOAR"))
+	fmt.Fprintln(output)
 }
 
-func printUsage(output io.Writer, flagSet *flag.FlagSet, withCommandline bool) {
+func heading(text string, colors twin.ColorType) string {
+	style := twin.StyleDefault.WithAttr(twin.AttrItalic)
+	prefix := style.RenderUpdateFrom(twin.StyleDefault, colors)
+	suffix := twin.StyleDefault.RenderUpdateFrom(style, colors)
+	return prefix + text + suffix
+}
+
+func printUsage(flagSet *flag.FlagSet, colors twin.ColorType) {
 	// This controls where PrintDefaults() prints, see below
-	flagSet.SetOutput(output)
+	flagSet.SetOutput(os.Stdout)
 
 	// FIXME: Log if any printouts fail?
 
-	if withCommandline {
-		printCommandline(output)
-	}
-
-	_, _ = fmt.Fprintln(output, "Usage:")
-	_, _ = fmt.Fprintln(output, "  moar [options] <file>")
-	_, _ = fmt.Fprintln(output, "  ... | moar")
-	_, _ = fmt.Fprintln(output, "  moar < file")
-	_, _ = fmt.Fprintln(output)
-	_, _ = fmt.Fprintln(output, "Shows file contents. Compressed files will be transparently decompressed.")
-	_, _ = fmt.Fprintln(output, "Input is expected to be (possibly compressed) UTF-8 encoded text. Invalid /")
-	_, _ = fmt.Fprintln(output, "non-printable characters are by default rendered as '?'.")
-	_, _ = fmt.Fprintln(output)
-	_, _ = fmt.Fprintln(output, "More information + source code:")
-	_, _ = fmt.Fprintln(output, "  <https://github.com/walles/moar#readme>")
-	_, _ = fmt.Fprintln(output)
-	_, _ = fmt.Fprintln(output, "Environment:")
+	fmt.Println(heading("Usage", colors))
+	fmt.Println("  moar [options] <file>")
+	fmt.Println("  ... | moar")
+	fmt.Println("  moar < file")
+	fmt.Println()
+	fmt.Println("Shows file contents. Compressed files will be transparently decompressed.")
+	fmt.Println("Input is expected to be (possibly compressed) UTF-8 encoded text. Invalid /")
+	fmt.Println("non-printable characters are by default rendered as '?'.")
+	fmt.Println()
+	fmt.Println("More information + source code:")
+	fmt.Println("  <https://github.com/walles/moar#readme>")
+	fmt.Println()
+	fmt.Println(heading("Environment", colors))
 
 	moarEnv := os.Getenv("MOAR")
 	if len(moarEnv) == 0 {
-		_, _ = fmt.Fprintln(output, "  Additional options are read from the MOAR environment variable if set.")
-		_, _ = fmt.Fprintln(output, "  But currently, the MOAR environment variable is not set.")
+		fmt.Println("  Additional options are read from the MOAR environment variable if set.")
+		fmt.Println("  But currently, the MOAR environment variable is not set.")
 	} else {
-		_, _ = fmt.Fprintln(output, "  Additional options are read from the MOAR environment variable.")
-		_, _ = fmt.Fprintf(output, "  Current setting: MOAR=\"%s\"\n", moarEnv)
+		fmt.Println("  Additional options are read from the MOAR environment variable.")
+		fmt.Printf("  Current setting: MOAR=\"%s\"\n", moarEnv)
 	}
 
-	printUsageEnvVar(output, "LESS_TERMCAP_md", "man page bold style")
-	printUsageEnvVar(output, "LESS_TERMCAP_us", "man page underline style")
-	printUsageEnvVar(output, "LESS_TERMCAP_so", "search hits and footer style")
+	printUsageEnvVar("LESS_TERMCAP_md", "man page bold style", colors)
+	printUsageEnvVar("LESS_TERMCAP_us", "man page underline style", colors)
+	printUsageEnvVar("LESS_TERMCAP_so", "search hits and footer style", colors)
 
 	absMoarPath, err := absLookPath(os.Args[0])
 	if err == nil {
@@ -91,24 +121,24 @@ func printUsage(output io.Writer, flagSet *flag.FlagSet, withCommandline bool) {
 		}
 		if absPagerValue != absMoarPath {
 			// We're not the default pager
-			_, _ = fmt.Fprintln(output)
-			_, _ = fmt.Fprintln(output, "Making moar your default pager:")
-			_, _ = fmt.Fprintln(output, "  Put the following line in your ~/.bashrc, ~/.bash_profile or ~/.zshrc")
-			_, _ = fmt.Fprintln(output, "  and moar will be used as the default pager in all new terminal windows:")
-			_, _ = fmt.Fprintln(output)
-			_, _ = fmt.Fprintf(output, "     export PAGER=%s\n", getMoarPath())
+			fmt.Println()
+			fmt.Println(heading("Making moar Your Default Pager", colors))
+			fmt.Println("  Put the following line in your ~/.bashrc, ~/.bash_profile or ~/.zshrc")
+			fmt.Println("  and moar will be used as the default pager in all new terminal windows:")
+			fmt.Println()
+			fmt.Printf("     export PAGER=%s\n", getMoarPath())
 		}
 	} else {
 		log.Warn("Unable to find moar binary ", err)
 	}
 
-	_, _ = fmt.Fprintln(output)
-	_, _ = fmt.Fprintln(output, "Options:")
+	fmt.Println()
+	fmt.Println(heading("Options", colors))
 
 	flagSet.PrintDefaults()
 
-	_, _ = fmt.Fprintln(output, "  +1234")
-	_, _ = fmt.Fprintln(output, "    \tImmediately scroll to line 1234")
+	fmt.Println("  +1234")
+	fmt.Println("    \tImmediately scroll to line 1234")
 }
 
 // "moar" if we're in the $PATH, otherwise an absolute path
@@ -181,14 +211,14 @@ func parseLexerOption(lexerOption string) (chroma.Lexer, error) {
 	)
 }
 
-func parseStyleOption(styleOption string) (chroma.Style, error) {
+func parseStyleOption(styleOption string) (*chroma.Style, error) {
 	style, ok := styles.Registry[styleOption]
 	if !ok {
-		return *styles.Fallback, fmt.Errorf(
+		return &chroma.Style{}, fmt.Errorf(
 			"Pick a style from here: https://xyproto.github.io/splash/docs/longer/all.html")
 	}
 
-	return *style, nil
+	return style, nil
 }
 
 func parseColorsOption(colorsOption string) (twin.ColorType, error) {
@@ -404,9 +434,9 @@ func main() {
 
 	wrap := flagSet.Bool("wrap", false, "Wrap long lines")
 	follow := flagSet.Bool("follow", false, "Follow piped input just like \"tail -f\"")
-	style := flagSetFunc(flagSet,
-		"style", *styles.Registry["native"],
-		"Highlighting style from https://xyproto.github.io/splash/docs/longer/all.html", parseStyleOption)
+	styleOption := flagSetFunc(flagSet,
+		"style", nil,
+		"Highlighting `style` from https://xyproto.github.io/splash/docs/longer/all.html", parseStyleOption)
 	lexer := flagSetFunc(flagSet,
 		"lang", nil,
 		"File contents, used for highlighting. Mime type or file extension (\"html\"). Default is to guess by filename.", parseLexerOption)
@@ -423,7 +453,7 @@ func main() {
 	quitIfOneScreen := flagSet.Bool("quit-if-one-screen", false, "Don't page if contents fits on one screen")
 	noClearOnExit := flagSet.Bool("no-clear-on-exit", false, "Retain screen contents when exiting moar")
 	statusBarStyle := flagSetFunc(flagSet, "statusbar", m.STATUSBAR_STYLE_INVERSE,
-		"Status bar style: inverse, plain or bold", parseStatusBarStyle)
+		"Status bar `style`: inverse, plain or bold", parseStatusBarStyle)
 	unprintableStyle := flagSetFunc(flagSet, "render-unprintable", textstyles.UnprintableStyleHighlight,
 		"How unprintable characters are rendered: highlight or whitespace", parseUnprintableStyle)
 	scrollLeftHint := flagSetFunc(flagSet, "scroll-left-hint",
@@ -432,12 +462,12 @@ func main() {
 	scrollRightHint := flagSetFunc(flagSet, "scroll-right-hint",
 		twin.NewCell('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
 		"Shown when view can scroll right. One character with optional ANSI highlighting.", parseScrollHint)
-	shift := flagSetFunc(flagSet, "shift", 16, "Horizontal scroll amount >=1, defaults to 16", parseShiftAmount)
+	shift := flagSetFunc(flagSet, "shift", 16, "Horizontal scroll `amount` >=1, defaults to 16", parseShiftAmount)
 	mouseMode := flagSetFunc(
 		flagSet,
 		"mousemode",
 		twin.MouseModeAuto,
-		"Mouse mode: auto, select or scroll: https://github.com/walles/moar/blob/master/MOUSE.md",
+		"Mouse `mode`: auto, select or scroll: https://github.com/walles/moar/blob/master/MOUSE.md",
 		parseMouseMode,
 	)
 
@@ -455,7 +485,7 @@ func main() {
 	err = flagSet.Parse(remainingArgs)
 	if err != nil {
 		if err == flag.ErrHelp {
-			printUsage(os.Stdout, flagSet, false)
+			printUsage(flagSet, *terminalColorsCount)
 			return
 		}
 
@@ -463,12 +493,11 @@ func main() {
 		if strings.HasPrefix(errorText, "invalid value") {
 			errorText = strings.Replace(errorText, ": ", "\n\n", 1)
 		}
+
 		boldErrorMessage := "\x1b[1m" + errorText + "\x1b[m"
 		fmt.Fprintln(os.Stderr, "ERROR:", boldErrorMessage)
 		fmt.Fprintln(os.Stderr)
-
 		printCommandline(os.Stderr)
-
 		fmt.Fprintln(os.Stderr, "For help, run: \x1b[1mmoar --help\x1b[m")
 
 		os.Exit(1)
@@ -491,9 +520,10 @@ func main() {
 	})
 
 	if len(flagSet.Args()) > 1 {
-		fmt.Fprintln(os.Stderr, "ERROR: Expected exactly one filename, or data piped from stdin, got:", flagSet.Args())
+		fmt.Fprintln(os.Stderr, "ERROR: Expected exactly one filename, or data piped from stdin")
 		fmt.Fprintln(os.Stderr)
-		printUsage(os.Stderr, flagSet, true)
+		printCommandline(os.Stderr)
+		fmt.Fprintln(os.Stderr, "For help, run: \x1b[1mmoar --help\x1b[m")
 
 		os.Exit(1)
 	}
@@ -517,7 +547,8 @@ func main() {
 	if inputFilename == nil && !stdinIsRedirected {
 		fmt.Fprintln(os.Stderr, "ERROR: Filename or input pipe required")
 		fmt.Fprintln(os.Stderr)
-		printUsage(os.Stderr, flagSet, true)
+		printCommandline(os.Stderr)
+		fmt.Fprintln(os.Stderr, "For help, run: \x1b[1mmoar --help\x1b[m")
 		os.Exit(1)
 	}
 
@@ -558,13 +589,44 @@ func main() {
 		formatter = formatters.TTY16m
 	}
 
+	var style chroma.Style
+	if *styleOption == nil {
+		t0 := time.Now()
+		select {
+		case event := <-screen.Events():
+			// Event received, let's see if it's the one we want
+			switch ev := event.(type) {
+
+			case twin.EventTerminalBackgroundDetected:
+				log.Debug("Terminal background color detected as ", ev.Color, " after ", time.Since(t0))
+
+				distanceToBlack := ev.Color.Distance(twin.NewColor24Bit(0, 0, 0))
+				distanceToWhite := ev.Color.Distance(twin.NewColor24Bit(255, 255, 255))
+				if distanceToBlack < distanceToWhite {
+					style = *styles.Get(defaultDarkTheme)
+				} else {
+					style = *styles.Get(defaultLightTheme)
+				}
+
+			default:
+				log.Debug("Expected terminal background color event but got ", ev, " after ", time.Since(t0), " putting back and giving up")
+				screen.Events() <- event
+			}
+
+		case <-time.After(5 * time.Millisecond):
+			log.Debug("Terminal background color still not detected after ", time.Since(t0), ", giving up")
+		}
+	} else {
+		style = **styleOption
+	}
+
 	var reader *m.Reader
 	if stdinIsRedirected {
 		// Display input pipe contents
-		reader = m.NewReaderFromStream("", os.Stdin, *style, formatter, *lexer)
+		reader = m.NewReaderFromStream("", os.Stdin, style, formatter, *lexer)
 	} else {
 		// Display the input file contents
-		reader, err = m.NewReaderFromFilename(*inputFilename, *style, formatter, *lexer)
+		reader, err = m.NewReaderFromFilename(*inputFilename, style, formatter, *lexer)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "ERROR: %v\n", err)
 			os.Exit(1)
@@ -589,7 +651,7 @@ func main() {
 		pager.TargetLineNumber = &reallyHigh
 	}
 
-	startPaging(pager, screen, style, &formatter)
+	startPaging(pager, screen, &style, &formatter)
 }
 
 // Define a generic flag with specified name, default value, and usage string.
